@@ -1,7 +1,7 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.103.2.18 2007/06/08 07:43:26 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/request.c,v 1.103.2.20 2008/02/11 23:24:12 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2007 The OpenLDAP Foundation.
+ * Copyright 1998-2008 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -482,7 +482,7 @@ ldap_new_connection( LDAP *ld, LDAPURLDesc *srvlist, int use_ldsb,
 							"ldap_new_connection %p: "
 							"unexpected response %d "
 							"from BIND request id=%d\n",
-							ld, ldap_msgtype( res ), msgid );
+							(void *)ld, ldap_msgtype( res ), msgid );
 						err = -1;
 						break;
 					}
