@@ -1,8 +1,8 @@
 /* syncrepl.c -- Replication Engine which uses the LDAP Sync protocol */
-/* $OpenLDAP: pkg/ldap/servers/slapd/syncrepl.c,v 1.168.2.52 2007/12/11 06:08:43 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/syncrepl.c,v 1.168.2.55 2008/02/13 18:06:05 quanah Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 2003-2007 The OpenLDAP Foundation.
+ * Copyright 2003-2008 The OpenLDAP Foundation.
  * Portions Copyright 2003 by IBM Corporation.
  * Portions Copyright 2003 by Howard Chu, Symas Corporation.
  * All rights reserved.
@@ -84,6 +84,7 @@ typedef struct syncinfo_s {
 	int					si_syncdata;
 	int					si_logstate;
 	int					si_conn_setup;
+	ber_int_t				si_msgid;
 	Avlnode				*si_presentlist;
 	LDAP				*si_ld;
 	LDAP_LIST_HEAD(np, nonpresent_entry) si_nonpresentlist;
@@ -315,7 +316,6 @@ ldap_sync_search(
 	BerElement *ber = (BerElement *)&berbuf;
 	LDAPControl c[2], *ctrls[3];
 	struct timeval timeout;
-	ber_int_t	msgid;
 	int rc;
 	int rhint;
 	char *base;
@@ -403,7 +403,7 @@ ldap_sync_search(
 
 	rc = ldap_search_ext( si->si_ld, base, scope, filter, attrs, attrsonly,
 		ctrls, NULL, si->si_tlimit > 0 ? &timeout : NULL,
-		si->si_slimit, &msgid );
+		si->si_slimit, &si->si_msgid );
 	ber_free_buf( ber );
 	return rc;
 }
@@ -634,8 +634,6 @@ do_syncrep2(
 	int	rc, err, i;
 	ber_len_t	len;
 
-	int rc_efree = 1;
-
 	struct berval	*psub;
 	Modifications	*modlist = NULL;
 
@@ -669,7 +667,7 @@ do_syncrep2(
 		tout_p = NULL;
 	}
 
-	while (( rc = ldap_result( si->si_ld, LDAP_RES_ANY, LDAP_MSG_ONE,
+	while (( rc = ldap_result( si->si_ld, si->si_msgid, LDAP_MSG_ONE,
 		tout_p, &res )) > 0 )
 	{
 		if ( slapd_shutdown ) {
@@ -718,19 +716,23 @@ do_syncrep2(
 						slap_parse_sync_cookie( &syncCookie, NULL );
 					}
 				}
+				rc = 0;
 				if ( si->si_syncdata && si->si_logstate == SYNCLOG_LOGGING ) {
-					entry = NULL;
 					modlist = NULL;
-					if ( syncrepl_message_to_op( si, op, msg ) == LDAP_SUCCESS &&
+					if (( rc = syncrepl_message_to_op( si, op, msg )) == LDAP_SUCCESS &&
 						!BER_BVISNULL( &syncCookie.ctxcsn ) ) {
 						syncrepl_updateCookie( si, op, psub, &syncCookie );
+					} else if ( rc == LDAP_NO_SUCH_OBJECT ) {
+						rc = LDAP_SYNC_REFRESH_REQUIRED;
+						si->si_logstate = SYNCLOG_FALLBACK;
+						ldap_abandon_ext( si->si_ld, si->si_msgid, NULL, NULL );
 					}
-				} else if ( syncrepl_message_to_entry( si, op, msg,
-					&modlist, &entry, syncstate ) == LDAP_SUCCESS ) {
-					rc_efree = syncrepl_entry( si, op, entry, &modlist,
-						syncstate, &syncUUID, &syncCookie_req, &syncCookie.ctxcsn );
-					if ( !BER_BVISNULL( &syncCookie.ctxcsn ) )
-					{
+				} else if (( rc = syncrepl_message_to_entry( si, op, msg,
+					&modlist, &entry, syncstate )) == LDAP_SUCCESS ) {
+					if (( rc = syncrepl_entry( si, op, entry, &modlist,
+						syncstate, &syncUUID, &syncCookie_req,
+						&syncCookie.ctxcsn )) == LDAP_SUCCESS &&
+						!BER_BVISNULL( &syncCookie.ctxcsn ) ) {
 						syncrepl_updateCookie( si, op, psub, &syncCookie );
 					}
 				}
@@ -738,10 +740,8 @@ do_syncrep2(
 				if ( modlist ) {
 					slap_mods_free( modlist, 1 );
 				}
-				if ( rc_efree && entry ) {
-					entry_free( entry );
-				}
-				entry = NULL;
+				if ( rc )
+					goto done;
 				break;
 
 			case LDAP_RES_SEARCH_REFERENCE:
@@ -1528,7 +1528,6 @@ syncrepl_message_to_entry(
 	}
 
 	e = ( Entry * ) ch_calloc( 1, sizeof( Entry ) );
-	*entry = e;
 	e->e_name = op->o_req_dn;
 	e->e_nname = op->o_req_ndn;
 
@@ -1604,9 +1603,10 @@ done:
 	if ( rc != LDAP_SUCCESS ) {
 		if ( e ) {
 			entry_free( e );
-			*entry = e = NULL;
+			e = NULL;
 		}
 	}
+	*entry = e;
 
 	return rc;
 }
@@ -1669,7 +1669,6 @@ syncrepl_entry(
 	AttributeAssertion ava = { NULL, BER_BVNULL };
 #endif
 	int rc = LDAP_SUCCESS;
-	int ret = LDAP_SUCCESS;
 
 	struct berval pdn = BER_BVNULL;
 	dninfo dni = {0};
@@ -1838,15 +1837,15 @@ retry_add:;
 			switch ( rs_add.sr_err ) {
 			case LDAP_SUCCESS:
 				be_entry_release_w( op, entry );
-				ret = 0;
+				entry = NULL;
 				break;
 
 			case LDAP_REFERRAL:
 			/* we assume that LDAP_NO_SUCH_OBJECT is returned 
 			 * only if the suffix entry is not present */
 			case LDAP_NO_SUCH_OBJECT:
-				syncrepl_add_glue( op, entry );
-				ret = 0;
+				rc = syncrepl_add_glue( op, entry );
+				entry = NULL;
 				break;
 
 			/* if an entry was added via syncrepl_add_glue(),
@@ -1882,7 +1881,8 @@ retry_add:;
 					cb2.sc_response = dn_callback;
 					cb2.sc_private = &dni;
 
-					be->be_search( &op2, &rs2 );
+					rc = be->be_search( &op2, &rs2 );
+					if ( rc ) goto done;
 
 					retry = 0;
 					slap_op_time( &op->o_time, &op->o_tincr );
@@ -1894,7 +1894,6 @@ retry_add:;
 				Debug( LDAP_DEBUG_ANY,
 					"syncrepl_entry: rid %03d be_add failed (%d)\n",
 					si->si_rid, rs_add.sr_err, 0 );
-				ret = 1;
 				break;
 			}
 			goto done;
@@ -1928,7 +1927,6 @@ retry_add:;
 				op->o_req_dn = entry->e_name;
 				op->o_req_ndn = entry->e_nname;
 			} else {
-				ret = 1;
 				goto done;
 			}
 			if ( dni.wasChanged )
@@ -1996,7 +1994,6 @@ retry_add:;
 					si->si_rid, rs_modify.sr_err, 0 );
 			}
 		}
-		ret = 1;
 		goto done;
 	case LDAP_SYNC_DELETE :
 		if ( !BER_BVISNULL( &dni.dn )) {
@@ -2024,13 +2021,11 @@ retry_add:;
 				}
 			}
 		}
-		ret = 0;
 		goto done;
 
 	default :
 		Debug( LDAP_DEBUG_ANY,
 			"syncrepl_entry: rid %03d unknown syncstate\n", si->si_rid, 0, 0 );
-		ret = 1;
 		goto done;
 	}
 
@@ -2048,8 +2043,10 @@ done :
 	if ( !BER_BVISNULL( &dni.dn ) ) {
 		op->o_tmpfree( dni.dn.bv_val, op->o_tmpmemctx );
 	}
+	if ( entry )
+		entry_free( entry );
 	BER_BVZERO( &op->o_csn );
-	return ret;
+	return rc;
 }
 
 static struct berval gcbva[] = {
@@ -2222,7 +2219,7 @@ syncrepl_del_nonpresent(
 	return;
 }
 
-void
+int
 syncrepl_add_glue(
 	Operation* op,
 	Entry *e )
@@ -2332,6 +2329,10 @@ syncrepl_add_glue(
 		} else {
 		/* incl. ALREADY EXIST */
 			entry_free( glue );
+			if ( rs_add.sr_err != LDAP_ALREADY_EXISTS ) {
+				entry_free( e );
+				return rc;
+			}
 		}
 
 		/* Move to next child */
@@ -2362,7 +2363,7 @@ syncrepl_add_glue(
 		entry_free( e );
 	}
 
-	return;
+	return rc;
 }
 
 static void
